import requests as lmao
import json as lmao2
from typing import Literal as lmao3
import random
from . import errors


# Proxies are your choice
class GeneratedImage:
    def __init__(self, returns: dict): 
        self.status =  returns.get("status")
        self.image_url = returns.get("image_url")
        self.request_id = returns.get("request_id")
        self.inference_status: dict = returns.get("inference_status")
        self.status = self.inference_status.get("status")
        self.runtime_ms: int = self.inference_status.get("runtime_ms")
        self.cost: float = self.inference_status.get("cost")
        self.tokens_generated: int = self.inference_status.get("tokens_generated")
        self.tokens_input: int = self.inference_status.get("tokens_input")
class GeneratedText:
    def __init__(self, returns: dict):
            self.id: str = returns.get("id")
            self.object: str = returns.get("object")
            self.created: int = returns.get("created")
            self.model: str = returns.get("model")
            choices = returns.get("choices", [])
            if choices:
                first_choice = choices[0]
                self.index: int = first_choice.get("index")
                self.message: dict = first_choice.get("message", {})
                self.role: str = self.message.get("role")
                self.content: str = self.message.get("content")
                self.finish_reason: str = first_choice.get("finish_reason")
            self.usage: dict = returns.get("usage", {})
            self.prompt_tokens: int = self.usage.get("prompt_tokens")
            self.completion_tokens: int = self.usage.get("completion_tokens")
            self.total_tokens: int = self.usage.get("total_tokens")
            self.estimated_cost: float = self.usage.get("estimated_cost")

class Model:
    def __init__(self, author: str, model: str, api_key: str = None, openai_completion: bool = True, proxy: str = None):
        self.author: str = author
        self.model: str = model
        self.api_key: str = api_key
        self.url = "https://api.deepinfra.com"
        self.openai_completion: bool = openai_completion
        self.proxy: str = proxy
        self.headers = {
            "Content-Type": "application/json",
        }
        if self.api_key:
            self.headers += {
                "Authorization": f"Bearer {self.api_key}"
            }
    def generate_text(self, prompt: str, system_prompt: str = "You are a helpful assistant", max_tokens: int = 512, temperature: float = 0.7, top_p: float = 0.9, min_p: float = 0, top_k: int = 0, repetition_penalty: float = 1, stop: list = [], response_format: lmao3["json_object", "none"] = "none", prescence_penalty: float = 0, frequencey_penalty: float = 0, seed: int = random.randint(0, 9223372036854776000)) -> GeneratedText:
        payload = {
            "model": f"{self.author}/{self.model}",
            "messages": [
                {
                    "role": "system",
                    "content": system_prompt
                },
                {
                    "role": "user",
                    "content": prompt
                }
            ],
            "stream": False,
            "max_tokens": max_tokens,
            "temperature": temperature,
            "top_p": top_p,
            "min_p": min_p,
            "repetition_penalty": repetition_penalty,
            "stop": stop,
            "top_k": top_k,
            "response_format": {
                "type": response_format
            },
            "presence_penalty": prescence_penalty,
            "frequency_penalty": frequencey_penalty,
            "seed": seed
        }
        response = lmao.post(f"{self.url}/v1/openai/chat/completions", headers=self.headers, json=payload)
        if max_tokens > 1000000:
            raise ValueError("Max tokens cannot exceed 1000000")
        if max_tokens < 0:
            raise ValueError("Max tokens cannot be less than 0")
        if temperature > 1:
            raise ValueError("Temperature cannot exceed 1")
        if temperature < 0:
            raise ValueError("Temperature cannot be less than 0")
        if top_p > 1:
            raise ValueError("Top P cannot exceed 1")
        if top_p < 0:
            raise ValueError("Top P cannot be less than 0")
        if min_p > 1:
            raise ValueError("Min P cannot exceed 1")
        if min_p < 0:
            raise ValueError("Min P cannot be less than 0")
        if top_k > 0:
            raise ValueError("Top K cannot be less than 0")
        if repetition_penalty > 5:
            raise ValueError("Repetition penalty cannot exceed 5")
        if repetition_penalty < 0.01:
            raise ValueError("Repetition penalty cannot be less than 0.01")
        if response_format not in ["json_object", "none"]:
            raise ValueError("Response format must be either json_object or none")
        if prescence_penalty > 2:
            raise ValueError("Presence penalty cannot exceed 2")
        if prescence_penalty < -2:
            raise ValueError("Presence penalty cannot be less than -2")
        if frequencey_penalty > 2:
            raise ValueError("Frequency penalty cannot exceed 2")
        if frequencey_penalty < -2:
            raise ValueError("Frequency penalty cannot be less than -2")
        if seed > 9223372036854776000:
            raise ValueError("Seed cannot exceed 9223372036854776000")
        if seed < 0:
            raise ValueError("Seed cannot be less than 0")
        
        match response.status_code:
            case 200:
                return GeneratedText(response.json())
            case 400:
                raise ValueError("Bad Request")
            case 403:
                raise errors.InvalidAPIKey("An API key is required for using this model.")
            case 404:
                raise errors.NoModelsFoundError("Model was not found. Check the author/model name or make sure you are using the correct function for the model.")
            case 429:
                raise ValueError("Rate limit exceeded. Please try again later.")
            case 500:
                raise ValueError("Internal Server Error")
            
    def generate_image(self, prompt: str, width: int = 1024, height: int = 1024, prompt_upsampling: bool = True, seed: int = random.randint(0, 2147483647), safety_tolerance: int = None):
        url = f"{self.url}/v1/inference/{self.author}/{self.model}"
        payload = {
            "prompt": prompt,
            "width": width,
            "height": height,
            "prompt_upsampling": prompt_upsampling,
            "seed": seed,
            "safety_tolerance": safety_tolerance
        }

        if prompt is None:
            raise ValueError("Prompt cannot be None")
        if width < 256:
            raise ValueError("Width cannot be less than 256")
        if height < 256:
            raise ValueError("Height cannot be less than 256")
        if width > 1440:
            raise ValueError("Width cannot exceed 1440")
        if height > 1440:
            raise ValueError("Height cannot exceed 1440")
        if safety_tolerance is not None:
            if safety_tolerance < 0:
                raise ValueError("Safety tolerance cannot be less than 0")
            if safety_tolerance > 2:
                raise ValueError("Safety tolerance cannot exceed 100")
        if seed < 0:
            raise ValueError("Seed cannot be less than 0")
        if seed > 2147483647:
            raise ValueError("Seed cannot exceed 2147483647")
        if isinstance(prompt_upsampling, bool) is False:
            raise ValueError("Prompt upsampling must be a boolean value")

        response = lmao.post(url, headers=self.headers, json=payload, proxies=self.proxy)
        match response.status_code:
            case 200:
                return GeneratedImage(response.json())
            case 400:
                raise ValueError("Bad Request")
            case 403:
                raise errors.InvalidAPIKey("An API key is required for using this model.")
            case 404:
                raise errors.NoModelsFoundError("Model was not found. Check the author/model name or make sure you are using the correct function for the model.")
            case 429:
                raise ValueError("Rate limit exceeded. Please try again later.")
            case 500:
                raise ValueError("Internal Server Error")
            case 402:
                raise errors.PaymentIsRequired("Payment is required to use this model.")
            case 401:
                raise errors.InvalidAPIKey("Invalid API Key. Please check your API key.")

class DeepInfra:
    def __init__(self, api_key: str = None, openai_completion: bool = True, proxy: str = None):
        """
        ## Unofficial Wrapper for Deep Infra.
        """
        self.api_key: str = api_key
        self.url = "https://api.deepinfra.com"
        self.openai_completion: bool = openai_completion
        self.proxy: str = proxy
    def model(self, author: str, model: str) -> Model:
        return Model(author, model, self.api_key, self.openai_completion, proxy=self.proxy)
    



